function [T2] = multiatest(r,rboots,sboots,gboots,K1) 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MATLAB code: Section 11.6.1 (Quantile residuals)
% File: multiatest.m
% Coded by: JDG (converted from GAUSS code written by Leena Kalliovirta) 
%
% Autocorrelation test statistic based on MULTIVARIATE (dimension of the data is 
% denoted by "m") quantile residuals (QRs).  
%               
% INPUT:     
% r      = vector of quantile residuals (T * m).        
% rboots = simulated vector of multivariate QRs using estimated model (Tboots * m) 
%          with Tboots some large number (say Tboots = 20,000).
% sboots = simulated matrix of scores of multivariate QRs using estimated model 
%          (Tboots * k) with k the number of columns of sboots.                                 
% gboots = simulated matrix of derivatives of QRs using estimated models 
%          (Tboots * mk).                                    
% K1     = number of lags used in the autocorrelation test. 
%            
% OUTPUT: 
% T2     = scalar value of the test statistic, follows a Chi^{2}(m^2 K1) 
%          distribution under H0: no autocorrelation in quantile residuals.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
T      = length(r);
Tboots = length(rboots);
[~,k]  = size(sboots); 
[~,m]  = size(r);

warning('off','all')  
warning
% Disable all warnings. Replace 'off' by 'on' to enable
% Warning: Matrix "cov" may be close to singular or badly scaled.
%          Results may be inaccurate. 

cov       = inv((sboots'*sboots)/Tboots);
gu        = zeros(T-K1,K1*m^2);
guboots   = zeros(Tboots-K1,K1*m^2);

for t1=1:(Tboots-K1)
  helpboots     = ((rboots(t1+K1,:))'*rboots(t1+K1-1,:));
  for i=2:K1
    helpboots   = [helpboots;((rboots(t1+K1,:))'*rboots(t1+K1-i,:))];
  end; 
  guboots(t1,:) = reshape(helpboots,[K1*m^2,1]);
end;

for t1=1:(T-K1)
  help = ((r(t1+K1,:))'*r(t1+K1-1,:));
  for i=2:K1
    help = [help;(r(t1+K1,:))'*r(t1+K1-i,:)];
  end;
  gu(t1,:) = reshape(help,[K1*m^2,1]);
end;

RR     = mean(gu);
Sigma2 = (horzcat(sboots(K1+1:Tboots,:),guboots))'...
         *(horzcat(sboots(K1+1:Tboots,:), guboots))/(Tboots-K1);
G2     = zeros(K1*m^2,k);
for j=1:K1
  for b=1:m
    for a=1:m
    rmat1 = rboots((K1+1-j):(Tboots-j),b);  
    repr1 = repmat(rmat1,1,k);              % repeat copies of array
    Crg1  = repr1.*gboots((K1+1):Tboots,((a-1)*k+1):a*k);
    
    rmat2 = rboots((K1+1):Tboots,a);
    repr2 = repmat(rmat2,1,k);              % repeat copies of array
    Crg2  = repr2.*gboots((K1+1-j):(Tboots-j),((b-1)*k+1):b*k);
    
    G2((j-1)*m^2+(b-1)*m+a,:) = (mean(Crg1)+mean(Crg2))';
    end;
  end;    
end;

O2 = [G2*cov,eye(K1*m^2)]*Sigma2*[G2*cov,eye(K1*m^2)]';
T2 = T*(RR*inv(O2)*RR');
